package com.ibm.etools.portal.portletexamples.content.jsr;

import java.io.*;
import java.net.MalformedURLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.portlet.*;

import com.ibm.portal.portlet.service.contentaccess.ContentAccessService;
import com.ibm.portal.portlet.service.*;

/**
 *
 * A sample portlet to demonstrate remote content access using ContentAccessService for JSR 168 API
 * 
 */
public class ContentAccessServicePortlet extends GenericPortlet {

	public static final String JSP_FOLDER    = "/jsp/";    // JSP folder name //$NON-NLS-1$

	public static final String VIEW_JSP      = "ContentAccessServicePortletView";         // JSP file name to be rendered on the view mode //$NON-NLS-1$
	public static final String EDIT_JSP      = "ContentAccessServicePortletEdit";         // JSP file name to be rendered on the edit mode //$NON-NLS-1$
	public static final String HELP_JSP      = "ContentAccessServicePortletHelp";         // JSP file name to be rendered on the help mode //$NON-NLS-1$
	public static final String CONFIG_JSP    = "ContentAccessServicePortletConfig";       // JSP file name to be rendered on the configure mode //$NON-NLS-1$
	public static final String ERROR_JSP     = "ContentAccessServicePortletError";   	  // JSP file name to be rendered on the configure mode //$NON-NLS-1$

	public static final String HELP_BEAN     = "ContentAccessServicePortletHelpBean";     // Bean name for the help mode request //$NON-NLS-1$

	public static final String SESSION_BEAN  = "ContentAccessServicePortletSessionBean";  // Bean name for the portlet session //$NON-NLS-1$

	public static final String EDIT_FORM_ACTION      	= "changeOption";     // Action name for the edit form //$NON-NLS-1$
	public static final String CONFIG_FORM_ACTION      	= "changeURI";     // Action name for the config form //$NON-NLS-1$
	public static final String CANCEL           = "cancelURI";         // Parameter name for general cancel button //$NON-NLS-1$
	public static final String URL              = "url";           // Parameter name for config param //$NON-NLS-1$
	public static final String FILTER_OPTION    = "filterOption";           // Parameter name for edit param //$NON-NLS-1$
	public static final String DIRECT    		= "direct"; //$NON-NLS-1$
	public static final String NUMBER_COUNT   	= "numberCount"; //$NON-NLS-1$
	public static final String CURRENT_URL   	= "currentURL"; //$NON-NLS-1$
	public static final String ERROR   			= "error"; //$NON-NLS-1$

	private static final PortletMode CUSTOM_CONFIG_MODE = new PortletMode("config"); //$NON-NLS-1$
	
	private PortletContext context;
	private ContentAccessService cas = null;

	public void init(PortletConfig config) throws PortletException {
		super.init(config);
		context = config.getPortletContext();
		PortletServiceHome casHome = null;  	
		try {
			Context ctx = new InitialContext();
			casHome = (PortletServiceHome) ctx.lookup("portletservice/com.ibm.portal.portlet.service.contentaccess.ContentAccessService");
		} catch (NamingException ex) {
			context.log(Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_Invalid_url._4")); //$NON-NLS-1$
		}

		if (casHome != null) {
			try {
				cas = (ContentAccessService) casHome.getPortletService
					(com.ibm.portal.portlet.service.contentaccess.ContentAccessService.class);
			} catch (PortletServiceUnavailableException ex) {				
				context.log("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error___1"); //$NON-NLS-1$
			}
		}

	}

	/**
	 * Serve up the <code>view</code> mode.
	 * 
	 * @see javax.portlet.GenericPortlet#doView(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		// Set the MIME type for the render response
		response.setContentType(request.getResponseContentType());

		String errorMessage = (String) request.getAttribute(ERROR);
		if (errorMessage != null) {
			invokeErrorJSP(request, response, errorMessage, null);
			return;
		}
		// Check if portlet session exists
		ContentAccessServicePortletSessionBean sessionBean = getSessionBean(request);
		if (sessionBean == null) {
			errorMessage = Messages.getString("ContentAccessServicePortlet.<b>NO_PORTLET_SESSION_YET</b>_3"); //$NON-NLS-1$
			invokeErrorJSP(request, response, errorMessage, null);
			return;
		}
	
		String filterOption = sessionBean.getFilterOption();
		if (filterOption == null)
			filterOption = DIRECT;
			
		PortletPreferences prefs = request.getPreferences();
		String url = prefs.getValue(URL, null);
		if (url == null || url.length() == 0) {
			errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_Invalid_url._4"); //$NON-NLS-1$
			invokeErrorJSP(request, response, errorMessage, null);
			return;
		}
		
		if (filterOption.equals(DIRECT)) {
			includeRemoteResource(url, request, response);
		} else {
			filterRemoteResource(url, request, response);
		}
	}

	/**
	 * Serve up the <code>edit</code> mode.
	 * 
	 * @see javax.portlet.GenericPortlet#doEdit(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
	 */
	public void doEdit(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		// Set the MIME type for the render response
		response.setContentType(request.getResponseContentType());

		PortletURL cancelUrl = response.createRenderURL();
		cancelUrl.setPortletMode(PortletMode.VIEW);
		request.setAttribute(CANCEL,cancelUrl.toString());

		PortletURL submitUrl = response.createActionURL();
		submitUrl.setPortletMode(PortletMode.VIEW);
		submitUrl.setParameter(EDIT_FORM_ACTION, EDIT_FORM_ACTION);
		request.setAttribute(EDIT_FORM_ACTION, submitUrl.toString());

		// Invoke the JSP to render
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getJspFilePath(request, EDIT_JSP));
		rd.include(request,response);
	}
	
	/**
	 * Serve up the <code>help</code> mode.
	 * 
	 * @see javax.portlet.GenericPortlet#doHelp(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
	 */
	protected void doHelp(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		// Set the MIME type for the render response
		response.setContentType(request.getResponseContentType());

		// Invoke the JSP to render
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getJspFilePath(request, HELP_JSP));
		rd.include(request,response);
	}

	/**
	 * Serve up the custom <code>config</code> mode.
	 */
	protected void doCustomConfigure(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		// Set the MIME type for the render response
		response.setContentType(request.getResponseContentType());

		PortletURL cancelUrl = response.createRenderURL();
		cancelUrl.setPortletMode(PortletMode.VIEW);
		request.setAttribute(CANCEL,cancelUrl.toString());

		PortletURL changeUrl = response.createActionURL();

		changeUrl.setParameter(CONFIG_FORM_ACTION, CONFIG_FORM_ACTION);
		request.setAttribute(CONFIG_FORM_ACTION, changeUrl.toString());
		
		PortletPreferences prefs = request.getPreferences();
		String currentUrl = prefs.getValue(URL, null);
		request.setAttribute(CURRENT_URL, currentUrl);

		// Invoke the JSP to render
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getJspFilePath(request, CONFIG_JSP));
		rd.include(request,response);
	}

	/**
	 * Override doDispatch method for handling custom portlet modes.
	 * 
	 * @see javax.portlet.GenericPortlet#doDispatch(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
	 */
	protected void doDispatch(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		if( !WindowState.MINIMIZED.equals(request.getWindowState()) ){
			PortletMode mode = request.getPortletMode();
			if( CUSTOM_CONFIG_MODE.equals(mode) ) {
				doCustomConfigure(request, response);
				return;
			}
		}
		super.doDispatch(request, response);
	}

	/**
	 * Process an action request.
	 * 
	 * @see javax.portlet.Portlet#processAction(javax.portlet.ActionRequest, javax.portlet.ActionResponse)
	 */
	public void processAction(ActionRequest request, ActionResponse response) throws PortletException, java.io.IOException {
		ContentAccessServicePortletSessionBean sessionBean = getSessionBean(request);
		
		if (request.getParameter(CONFIG_FORM_ACTION) != null) {
			String newUrl = request.getParameter(URL);
			String errorMessage = null;
			if (newUrl == null || newUrl.length() == 0) {
				errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error__Invalid_url_is_specified._14"); //$NON-NLS-1$
				context.log(errorMessage); 
				request.setAttribute(ERROR, errorMessage);
				return;
			}

			PortletPreferences prefs = request.getPreferences();
			prefs.setValue(URL, newUrl);
			
			try {
				prefs.store();
				response.setPortletMode(PortletMode.VIEW);
			} catch (IOException ex) {
				errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error__Storing_PortletSetting_failed._15"); //$NON-NLS-1$ 
				context.log(errorMessage, ex);
				request.setAttribute(ERROR, errorMessage);
			} catch (ValidatorException ex) {
				errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error__Storing_PortletSetting_failed._15"); //$NON-NLS-1$ 
				context.log(errorMessage, ex);
				request.setAttribute(ERROR, errorMessage);
			}
		} else if (request.getParameter(EDIT_FORM_ACTION) != null) {
			sessionBean.setFilterOption(request.getParameter(FILTER_OPTION));

		}
	}

	/**
	 * Get SessionBean.
	 * 
	 * @param request PortletRequest
	 * @return BasicPortletSessionBean
	 */
	private static ContentAccessServicePortletSessionBean getSessionBean(PortletRequest request) {
		PortletSession session = request.getPortletSession();
		if( session == null )
			return null;
		ContentAccessServicePortletSessionBean sessionBean = (ContentAccessServicePortletSessionBean)session.getAttribute(SESSION_BEAN);
		if( sessionBean == null ) {
			sessionBean = new ContentAccessServicePortletSessionBean();
			session.setAttribute(SESSION_BEAN,sessionBean);
		}
		return sessionBean;
	}

	/**
	 * Returns JSP file path.
	 * 
	 * @param request Render request
	 * @param jspFile JSP file name
	 * @return JSP file path
	 */
	private static String getJspFilePath(RenderRequest request, String jspFile) {
		String markup = request.getProperty("wps.markup");
		if( markup == null )
			markup = getMarkup(request.getResponseContentType());
		return JSP_FOLDER+markup+"/"+jspFile+"."+getJspExtension(markup);
	}
	
	/**
	 * Convert MIME type to markup name.
	 * 
	 * @param contentType MIME type
	 * @return Markup name
	 */
	private static String getMarkup(String contentType) {
		if( "text/vnd.wap.wml".equals(contentType) )
			return "wml";
		return "html";
	}

	/**
	 * Returns the file extension for the JSP file
	 * 
	 * @param markupName Markup name
	 * @return JSP extension
	 */
	private static String getJspExtension(String markupName) {
		return "jsp";
	}
	
	/**
	 * Access remote resources and render directly using ContentAccessService
	 * 
	 * @param cas
	 * @param url
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	private void includeRemoteResource(String url, RenderRequest request, RenderResponse response) 
	throws PortletException, IOException {
		String errorMessage = null;
		try {
			if (cas != null)
				cas.include(url, request, response, context);
		} catch (MalformedURLException ex) {
			errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error___1"); //$NON-NLS-1$
			invokeErrorJSP(request, response, errorMessage, ex);
		}
	}
	
	/**
	 * Access remote resources and filter the content using ContentAcessService
	 * 
	 * @param cas
	 * @param url
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	private void filterRemoteResource(String url, RenderRequest request, RenderResponse response) 
	throws PortletException, IOException {
		String errorMessage = null;
		InputStream in = null;
		int numberCount = -1;
		try {
			if (cas != null) {
				in = cas.getInputStream(url, request, response);
				numberCount = SimpleNumberCounter.count(in);
			}
		} catch (MalformedURLException ex) {
			errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error___1"); //$NON-NLS-1$
			invokeErrorJSP(request, response, errorMessage, ex);
			return;
		} catch (IOException ex) {
			errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error___1"); //$NON-NLS-1$
			invokeErrorJSP(request, response, errorMessage, ex);
			return;
		} finally {
			try {
				if (in != null)
					in.close();
			} catch (IOException ex) {
				errorMessage = Messages.getString("ContentAccessServicePortlet.ContentAccessServicePortlet_(JSR168)_error___1"); //$NON-NLS-1$
				invokeErrorJSP(request, response, errorMessage, ex);
				return;
			}
		}
		request.setAttribute(NUMBER_COUNT, String.valueOf(numberCount));
		request.setAttribute(CURRENT_URL, url);

		// Invoke the JSP to render
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getJspFilePath(request, VIEW_JSP));
		rd.include(request,response);
	}
	
	/**
	 * log error message and render error JSP
	 * 
	 * @param request
	 * @param response
	 * @param errorMessage
	 * @param exception
	 * @throws PortletException
	 * @throws IOException
	 */
	private void invokeErrorJSP(RenderRequest request, RenderResponse response, String errorMessage, Throwable exception) 
	throws PortletException, IOException {
		if (exception != null) {
			context.log(errorMessage, exception);
		} else {
			context.log(errorMessage);
		}
		request.setAttribute(ERROR, errorMessage);
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getJspFilePath(request, ERROR_JSP));
		rd.include(request, response);
	}
}
