/* @copyright module */
/*                                                                            */
/*  DISCLAIMER OF WARRANTIES:                                                 */
/*  -------------------------                                                 */
/*  The following [enclosed] code is sample code created by IBM Corporation.  */
/*  This sample code is provided to you solely for the purpose of assisting   */
/*  you in the development of your applications.                              */
/*  The code is provided "AS IS", without warranty of any kind. IBM shall     */
/*  not be liable for any damages arising out of your use of the sample code, */
/*  even if they have been advised of the possibility of such damages.        */
package com.ibm.etools.portal.portletexamples.cooperative.legacy;

import java.io.IOException;
import java.text.MessageFormat;

import org.apache.jetspeed.portlet.PortletAdapter;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.wps.logging.LogManager;
import com.ibm.wps.logging.Logger;

public class AccountsPortlet 
extends PortletAdapter
implements ActionListener {
    private static final String PREFIX = "";
    public static final String ACTION_NAME = PREFIX + "actionName";
    public static final String ACCOUNT_DETAILS = PREFIX + "accountDetails";
    public static final String ACCOUNT_ID_ENTRY = PREFIX + "orderIdEntry";
    public static final String ACCOUNT_ID = PREFIX + "orderId";
    public static final String ACCOUNT_DETAIL_BEAN = PREFIX + "accountDetailBean";

    private static Logger log = LogManager.getLogManager().getLogger(AccountsPortlet.class);

    /**
     * Process the portlet request, according to action 
     * to create protlet response
     *
     * @param request The Portlet request we are processing
     * @param response The Portlet response we are creating
     *
     */
    public void doView(PortletRequest request, PortletResponse response) 
    throws PortletException,IOException
    {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "doView");
        }
        AccountDetailBean adb = new AccountDetailBean();
        try {
            String action = (String) request.getPortletSession().getAttribute(ACTION_NAME);
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "doView", "action = " + action);
            }
            request.setAttribute(ACCOUNT_DETAIL_BEAN, adb);

            if (action == null || action.equals(ACCOUNT_ID_ENTRY)) {
                //Show initial page
                String actionURI = ShippingUtils.createSimpleActionURI(ACCOUNT_DETAILS, response);
                adb.setActionURI(actionURI);
                getPortletConfig().getContext().include("/jsp/html/AccountDetailEntry.jsp",
                                                        request, response);
            } else if (action.equals(ACCOUNT_DETAILS)) {

                String actionURI = ShippingUtils.createSimpleActionURI(ACCOUNT_DETAILS, response); 

                adb.setActionURI(actionURI);
                String accountId = (String) request.getPortletSession().getAttribute(ACCOUNT_ID);
                if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                    log.text(Logger.TRACE_MEDIUM, "doView", "account id: " + accountId);
                }
                AccountDetail ad = ShippingDB.getAccountDetail(accountId);

                if (ad != null) {
                    adb.setAccountDetail(ad);
                    getPortletConfig().getContext().include("/jsp/html/AccountsView.jsp",
                                                            request, response);
                } else {
                    //No match; print error page
                    if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                        log.text(Logger.TRACE_MEDIUM, "doView", "account record not found");
                    }
                    String msg = MessageFormat.format(Messages.getString("ERR_AccountsPortlet.0"), new String[] {accountId}); 
                    adb.setErrorMessage(msg);
                    getPortletConfig().getContext().include("/jsp/html/AccountDetailError.jsp", request, response);

                }
            }

        } catch (Exception exc) {
            //No match; print error page
            request.setAttribute(ACCOUNT_DETAIL_BEAN, adb);
            adb.setErrorMessage(Messages.getString("ERR_AccountsPortlet.1"));
            getPortletConfig().getContext().include("/jsp/html/AccountDetailError.jsp", request, response);
            log.text(Logger.ERROR, "doView", "unexpected exception", exc);
        }
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "doView");
        }
    }

    /**
     * Process ActionEvent, according to action name
     * to set request attribute
     *
     * @param event The ActionEvent we are processing
     *
     * @exception PortletException,IOException
     */  
    public void actionPerformed (ActionEvent event) {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "actionPerformed");
        }
        String actionName = event.getActionString();
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.text(Logger.TRACE_MEDIUM, "actionPerformed", "action = " + actionName);
        }
        PortletRequest request = event.getRequest();

        //An action causes the state to be modified
        ShippingUtils.setLastModified(request);

        if (actionName.equals(ACCOUNT_DETAILS)) {
            request.getPortletSession().setAttribute(ACTION_NAME, ACCOUNT_DETAILS);
            request.getPortletSession().setAttribute(ACCOUNT_ID, request.getParameter(ACCOUNT_ID));
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "actionPerformed", "accountId = " + request.getParameter(ACCOUNT_ID));
            }
        } else if (actionName.equals(ACCOUNT_ID_ENTRY)) {
            request.getPortletSession().setAttribute(ACTION_NAME, ACCOUNT_ID_ENTRY);
        }
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "actionPerformed");
        }
    }

    public long getLastModified(PortletRequest request) {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "getLastModified");
        }
        long lastModified =  ShippingUtils.getLastModified(request);
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "getLastModified", lastModified);
        }
        return lastModified;
    }

}

