/* @copyright module */
/*                                                                            */
/*  DISCLAIMER OF WARRANTIES:                                                 */
/*  -------------------------                                                 */
/*  The following [enclosed] code is sample code created by IBM Corporation.  */
/*  This sample code is provided to you solely for the purpose of assisting   */
/*  you in the development of your applications.                              */
/*  The code is provided "AS IS", without warranty of any kind. IBM shall     */
/*  not be liable for any damages arising out of your use of the sample code, */
/*  even if they have been advised of the possibility of such damages.        */
package com.ibm.etools.portal.portletexamples.cooperative.legacy;

import java.io.IOException;
import java.text.MessageFormat;

import org.apache.jetspeed.portlet.PortletAdapter;
import org.apache.jetspeed.portlet.PortletException;
import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.event.ActionEvent;
import org.apache.jetspeed.portlet.event.ActionListener;

import com.ibm.wps.logging.LogManager;
import com.ibm.wps.logging.Logger;

/*
 *
 * This portlet will get data from a mock database and 
 * display customer detail of a order 
 *
 */ 
public class CustomerPortlet 
extends PortletAdapter
implements ActionListener {
    private static final String PREFIX = "";
    public static final String ACTION_NAME = PREFIX + "actionName";
    public static final String CUSTOMER_DETAILS = PREFIX + "customerDetails";
    public static final String CUSTOMER_DETAILS_BY_NAME = PREFIX + "customerDetailsByName";
    public static final String CUSTOMER_ID_ENTRY = PREFIX + "customerIdEntry";
    public static final String CUSTOMER_ID = PREFIX + "customerId";
    public static final String CUSTOMER_NAME = PREFIX + "customerName";
    public static final String CUSTOMER_DETAIL_BEAN = PREFIX + "customerDetailBean";

    private static Logger log = LogManager.getLogManager().getLogger(CustomerPortlet.class);

    /**
     * Process the portlet request, according to action 
     * to create protlet response
     *
     * @param request The Portlet request we are processing
     * @param response The Portlet response we are creating
     *
     * @exception PortletException,IOException
     */
    public void doView(PortletRequest request, PortletResponse response)
    throws PortletException,IOException
    {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "doView");
        }

        CustomerDetailBean cdb = new CustomerDetailBean();
        try {

            String action = (String) request.getPortletSession().getAttribute(ACTION_NAME);
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "doView", "action = " + action);
            }
            request.setAttribute(CUSTOMER_DETAIL_BEAN, cdb);

            if (action == null || action.equals(CUSTOMER_ID_ENTRY)) {

                //Show initial page
                String actionURI = ShippingUtils.createSimpleActionURI(CUSTOMER_DETAILS, response);
                cdb.setActionURI(actionURI);
                getPortletConfig().getContext().include("/jsp/html/CustomerDetailEntry.jsp",
                                                        request, response);

            } else if (action.equals(CUSTOMER_DETAILS)) {

                String actionURI = ShippingUtils.createSimpleActionURI(CUSTOMER_DETAILS, response);  
                cdb.setActionURI(actionURI);
                String customerId = (String) request.getPortletSession().getAttribute(CUSTOMER_ID);
                if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                    log.text(Logger.TRACE_MEDIUM, "doView", "customerId = " + customerId);
                }
                CustomerDetail cd = ShippingDB.getCustomerDetail(customerId);
                if (cd != null) {
                    cdb.setCustomerDetail(cd);
                    getPortletConfig().getContext().include("/jsp/html/CustomerView.jsp",
                                                            request, response);
                } else {
                    //No match; print error page
                	String msg = MessageFormat.format(Messages.getString("ERR_CustomerPortlet.0"), new String[] {customerId}); //$NON-NLS-1$
                    cdb.setErrorMessage(msg);
                    getPortletConfig().getContext().include("/jsp/html/CustomerDetailError.jsp", request, response);
                }
            } else if (action.equals(CUSTOMER_DETAILS_BY_NAME)) {
                String actionURI = ShippingUtils.createSimpleActionURI(CUSTOMER_DETAILS, response);   
                cdb.setActionURI(actionURI);
                String customerName = (String) request.getPortletSession().getAttribute(CUSTOMER_NAME);
                CustomerDetail cd = ShippingDB.getCustomerDetailByName(customerName);
                if (cd != null) {
                    cdb.setCustomerDetail(cd);
                    getPortletConfig().getContext().include("/jsp/html/CustomerView.jsp",
                                                            request, response);
                } else {
                    //No match; print error page
                    if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                        log.text(Logger.TRACE_MEDIUM, "doView", "Customer record not found");
                    }
                    String msg = MessageFormat.format(Messages.getString("ERR_CustomerPortlet.1"), new String[] {customerName}); //$NON-NLS-1$
                    cdb.setErrorMessage(msg);
                    getPortletConfig().getContext().include("/jsp/html/CustomerDetailError.jsp", request, response);
                }
            }

        } catch (Exception exc) {
            //Exception; print error page 
            request.setAttribute(CUSTOMER_DETAIL_BEAN, cdb);
            cdb.setErrorMessage(Messages.getString("ERR_CustomerPortlet.2")); //$NON-NLS-1$
            getPortletConfig().getContext().include("/jsp/html/CustomerDetailError.jsp", request, response);
            log.text(Logger.ERROR, "doView", "unexpected exception", exc);
        }
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "doView");
        }
    }

    /**
     * Process ActionEvent, according to action name
     * to set request attribute
     *
     * @param event The ActionEvent we are processing
     *
     */  
    public void actionPerformed (ActionEvent event) 
    {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "actionPerformed");
        }

        String actionName = event.getActionString();

        PortletRequest request = event.getRequest();

        //An action causes the state to be modified
        ShippingUtils.setLastModified(request);

        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.text(Logger.TRACE_MEDIUM, "actionPerformed", "action = " + actionName);
        }
        if (actionName.equals(CUSTOMER_DETAILS)) {
            request.getPortletSession().setAttribute(ACTION_NAME, CUSTOMER_DETAILS);

            request.getPortletSession().setAttribute(CUSTOMER_ID, request.getParameter(CUSTOMER_ID));
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "actionPerformed", "customerId = " + request.getParameter(CUSTOMER_ID));
            }
        } else if (actionName.equals(CUSTOMER_DETAILS_BY_NAME)) {
            request.getPortletSession().setAttribute(ACTION_NAME, CUSTOMER_DETAILS_BY_NAME);
            request.getPortletSession().setAttribute(CUSTOMER_NAME, request.getParameter(CUSTOMER_NAME));
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "actionPerformed", "customerName = " + request.getParameter(CUSTOMER_NAME));
            }
        } else if (actionName.equals(CUSTOMER_ID_ENTRY)) {
            request.getPortletSession().setAttribute(ACTION_NAME, CUSTOMER_ID_ENTRY);
            if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
                log.text(Logger.TRACE_MEDIUM, "actionPerformed", "customerId = " + request.getParameter(CUSTOMER_ID));
            }
        }
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "actionPerformed");
        }
    }

    public long getLastModified(PortletRequest request) {
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.entry(Logger.TRACE_MEDIUM, "getLastModified");
        }
        long lastModified = ShippingUtils.getLastModified(request);
        if ( log.isLogging(Logger.TRACE_MEDIUM) ) {
            log.exit(Logger.TRACE_MEDIUM, "getLastModified", lastModified);
        }
        return lastModified;
    }
}
