/* @copyright module */
/*                                                                            */
/*  DISCLAIMER OF WARRANTIES:                                                 */
/*  -------------------------                                                 */
/*  The following [enclosed] code is sample code created by IBM Corporation.  */
/*  This sample code is provided to you solely for the purpose of assisting   */
/*  you in the development of your applications.                              */
/*  The code is provided "AS IS", without warranty of any kind. IBM shall     */
/*  not be liable for any damages arising out of your use of the sample code, */
/*  even if they have been advised of the possibility of such damages.        */
package com.ibm.etools.portal.portletexamples.cooperative.jsr;

import java.io.IOException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.ibm.portal.portlet.service.PortletServiceHome;
import com.ibm.portal.propertybroker.service.PropertyBrokerService;

public class CustomerPortlet 
       extends GenericPortlet
{
   private static final String PREFIX = "";
   public static final String ACTION_NAME = PREFIX + "actionName";
   public static final String CUSTOMER_DETAILS = PREFIX + "customerDetails";
   public static final String CUSTOMER_DETAILS_BY_NAME = PREFIX + "customerDetailsByName";
   public static final String CUSTOMER_ID_ENTRY = PREFIX + "customerIdEntry";
   public static final String CUSTOMER_ID = PREFIX + "customerId";
   public static final String CUSTOMER_NAME = PREFIX + "customerName";
   public static final String CUSTOMER_DETAIL_BEAN = PREFIX + "customerDetailBean";
   public static final String ACTION_NAME_PARAM = ShippingUtils.ACTION_NAME_PARAM;

   private static final String ENTRY_JSP = "CustomerDetailEntry.jsp"; //$NON-NLS-1$
   private static final String VIEW_JSP = "CustomerView.jsp"; //$NON-NLS-1$
   private static final String ERROR_JSP = "CustomerDetailError.jsp"; //$NON-NLS-1$
   
   boolean pbServiceAvailable = false;
   PropertyBrokerService pbService;
   public void init(PortletConfig config) throws PortletException {
       super.init(config);

      try {
           Context ctx = new InitialContext();
           PortletServiceHome serviceHome = (PortletServiceHome) ctx.lookup("portletservice/com.ibm.portal.propertybroker.service.PropertyBrokerService"); 
           pbService = (PropertyBrokerService) serviceHome.getPortletService(com.ibm.portal.propertybroker.service.PropertyBrokerService.class);
           pbServiceAvailable = true;
      }catch(Throwable t) {
           pbServiceAvailable = false;
           getPortletContext().log("CustomerPortlet could not find property broker service!");
      } 
   }

    public void doView(RenderRequest request, RenderResponse response)
    throws PortletException,IOException
    {
        getPortletContext().log("CustomerPortlet doView entry");
		// Set the MIME type for the render response
		response.setContentType(request.getResponseContentType());

        try {

	    String action = (String) request.getPortletSession().getAttribute(ACTION_NAME);
	    CustomerDetailBean cdb = new CustomerDetailBean();
	    request.setAttribute(CUSTOMER_DETAIL_BEAN, cdb);

	    if (action == null || action.equals(CUSTOMER_ID_ENTRY)) {

	        //Show initial page
                PortletURL actionURL = ShippingUtils.createSimpleActionURL(CUSTOMER_DETAILS, response);
                cdb.setActionURL(actionURL);
                getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, ENTRY_JSP)).include(request, response);

	    } else if (action.equals(CUSTOMER_DETAILS)) {

            PortletURL actionURL = ShippingUtils.createSimpleActionURL(CUSTOMER_DETAILS, response);
            cdb.setActionURL(actionURL);

	        String customerId = (String) request.getPortletSession().getAttribute(CUSTOMER_ID);
		CustomerDetail cd = ShippingDB.getCustomerDetail(customerId);
		if (cd != null) {
		    cdb.setCustomerDetail(cd);
                    getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, VIEW_JSP)).include(request, response);
                } else {
		    //No match; print error page
		    cdb.setErrorMessage("Customer Id " + customerId + " not found.");
                    getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, ERROR_JSP)).include(request, response);
		}
	    } else if (action.equals(CUSTOMER_DETAILS_BY_NAME)) {
                PortletURL actionURL = ShippingUtils.createSimpleActionURL(CUSTOMER_DETAILS, response);

                cdb.setActionURL(actionURL);
	        String customerName = (String) request.getPortletSession().getAttribute(CUSTOMER_NAME);
		CustomerDetail cd = ShippingDB.getCustomerDetailByName(customerName);
		if (cd != null) {
		    cdb.setCustomerDetail(cd);
                    getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, VIEW_JSP)).include(request, response);
                } else {
		    //No match; print error page
		    cdb.setErrorMessage("Customer name " + customerName + " not found.");
                    getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, ERROR_JSP)).include(request, response);
		}
	    }

        } catch (Exception exc) {
            getPortletContext().log("CustomerPortlet: An error occurred", exc);
        }
    }

    public void processAction (ActionRequest request, ActionResponse response) {
            getPortletContext().log("CustomerPortlet processAction entry");

            String actionName = request.getParameter("ACTION_NAME");
            if (actionName == null) {
                    actionName = "";
            }
            //An action causes the state to be modified
            ShippingUtils.setLastModified(request);
            if (actionName.equals(CUSTOMER_DETAILS)) {
                    request.getPortletSession().setAttribute(ACTION_NAME, CUSTOMER_DETAILS);
                    request.getPortletSession().setAttribute(CUSTOMER_ID, request.getParameter(CUSTOMER_ID));
            } else if (actionName.equals(CUSTOMER_DETAILS_BY_NAME)) {
                    request.getPortletSession().setAttribute(ACTION_NAME, CUSTOMER_DETAILS_BY_NAME);
                    request.getPortletSession().setAttribute(CUSTOMER_NAME, request.getParameter(CUSTOMER_NAME));
            } else if (actionName.equals(CUSTOMER_ID_ENTRY)) {
                    request.getPortletSession().setAttribute(ACTION_NAME, CUSTOMER_ID_ENTRY);
            }
    }
    public long getLastModified(PortletRequest request) {
        return ShippingUtils.getLastModified(request);
    
    }
}
