/* @copyright module */
/*                                                                            */
/*  DISCLAIMER OF WARRANTIES:                                                 */
/*  -------------------------                                                 */
/*  The following [enclosed] code is sample code created by IBM Corporation.  */
/*  This sample code is provided to you solely for the purpose of assisting   */
/*  you in the development of your applications.                              */
/*  The code is provided "AS IS", without warranty of any kind. IBM shall     */
/*  not be liable for any damages arising out of your use of the sample code, */
/*  even if they have been advised of the possibility of such damages.        */
package com.ibm.etools.portal.portletexamples.cooperative.jsr;

import java.io.IOException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.ibm.portal.portlet.service.PortletServiceHome;
import com.ibm.portal.propertybroker.service.PropertyBrokerService;

public class OrderDetailPortlet 
       extends GenericPortlet
{

   private static final String PREFIX = "";
   public static final String ACTION_NAME = PREFIX + "actionName";
   public static final String ORDER_DETAILS = PREFIX + "orderDetails";
   public static final String ORDER_ID_ENTRY = PREFIX + "orderIdEntry";
   public static final String ORDER_ID = PREFIX + "orderId";
   public static final String ORDER_DETAIL_BEAN = PREFIX + "orderDetailBean";
   public static final String ORDER_DETAIL = PREFIX + "orderDetail";
   public static final String TRACKING_ID = PREFIX + "trackingId";
   public static final String PBSERVICE = PREFIX + "pbService";
   public static final String PBSERVICEAVAILABLE = PREFIX + "pbServiceAvailable";
   public static final String ACTION_NAME_PARAM = ShippingUtils.ACTION_NAME_PARAM;
   
   private static final String ENTRY_JSP = "OrderDetailEntry.jsp"; //$NON-NLS-1$
   private static final String VIEW_JSP = "OrderDetailView.jsp"; //$NON-NLS-1$
   private static final String ERROR_JSP = "OrderDetailError.jsp"; //$NON-NLS-1$
   
   boolean pbServiceAvailable = false;
   PropertyBrokerService pbService = null;
   public void init(PortletConfig config) throws PortletException {
       super.init(config);

      try {
           Context ctx = new InitialContext();
           PortletServiceHome serviceHome = (PortletServiceHome) ctx.lookup("portletservice/com.ibm.portal.propertybroker.service.PropertyBrokerService"); 
           pbService = (PropertyBrokerService)serviceHome.getPortletService(com.ibm.portal.propertybroker.service.PropertyBrokerService.class);
           pbServiceAvailable = true;
      }catch(Throwable t) {
           getPortletContext().log("OrderDetailPortlet could not find property broker service!");
      } 
    }

    public void doView(RenderRequest request, RenderResponse response) 
    throws PortletException,IOException
    {
        getPortletContext().log("OrderDetailPortlet doView called");
		// Set the MIME type for the render response
		response.setContentType(request.getResponseContentType());
        try {

	        String action = (String) request.getPortletSession().getAttribute(ACTION_NAME);
                getPortletContext().log("OrderDetailPortlet doView ActionName=" + action);
	        OrderDetailBean odb = new OrderDetailBean();
	        request.setAttribute(ORDER_DETAIL_BEAN, odb);
                if (pbServiceAvailable == true) {
                    try {
                        odb.setTrackingIdActive(pbService.areWiresActive(request, TRACKING_ID));
                    } catch (Exception e) {
                        getPortletContext().log("OrderDetailPortlet: unexpected exception accessing property broker service", e);
                    }
                } 
			
	        if (action == null || action.equals(ORDER_ID_ENTRY)) {

	            //Show initial page
                PortletURL actionURL = ShippingUtils.createSimpleActionURL(ORDER_DETAILS, response);
                odb.setActionURL(actionURL);
                getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, ENTRY_JSP)).include(request, response);

	        } else if (action.equals(ORDER_DETAILS)) {

	            String orderId = (String) request.getPortletSession().getAttribute(ORDER_ID);
		        OrderDetail od = ShippingDB.getOrderDetail(orderId);

                PortletURL actionURL = ShippingUtils.createSimpleActionURL(ORDER_DETAILS, response);

		        odb.setActionURL(actionURL);

		        if (od != null) {
		            odb.setOrderDetail(od);
                    getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, VIEW_JSP)).include(request, response);
                } else {
		            //No match; print error page
	                odb.setErrorMessage("Order Id " + orderId + " not found.");
                    getPortletContext().getRequestDispatcher(ShippingUtils.getJspFilePath(request, ERROR_JSP)).include(request, response);
		        }
            }
       } catch (Exception exc) {
           getPortletContext().log("OrderDetailPortlet:  An error occurred", exc);
       }
    } 
    
    public void processAction (ActionRequest request, ActionResponse response) {
        getPortletContext().log("OrderDetail processAction called");

        String actionName = request.getParameter(ACTION_NAME_PARAM);
        if (actionName == null) {
            actionName = "";
        }
        getPortletContext().log("OrderDetail processAction, actionName = " + actionName);

        //An action causes the state to be modified
        ShippingUtils.setLastModified(request);


        if (actionName.equals(ORDER_DETAILS)) {
            String orderId = request.getParameter(ORDER_ID);
            getPortletContext().log("OrderDetail processAction, orderId from request = " + orderId);
            if (orderId == null) {
                orderId = (String)request.getPortletSession().getAttribute(ORDER_ID);
            }

            //We do this as tracking id is an out param in the init()
            //We write the tracking id in the request so it can be published by
            //the broker in the same event cycle
            if (orderId != null) {
                request.getPortletSession().setAttribute(ACTION_NAME, ORDER_DETAILS);
                request.getPortletSession().setAttribute(ORDER_ID, orderId);

                OrderDetail od = ShippingDB.getOrderDetail(orderId);
                if (od != null) {
                	request.getPortletSession().setAttribute(ORDER_DETAIL, od);
                    request.setAttribute(TRACKING_ID, od.getTrackingId());
                    getPortletContext().log("OrderDetail processAction ActionName is OrderDetail. TrackingId=" + od.getTrackingId());
                }
            }

        } else if (actionName.equals(ORDER_ID_ENTRY)) {
            request.getPortletSession().setAttribute(ACTION_NAME, ORDER_ID_ENTRY);
        }
    }

    public long getLastModified(PortletRequest request) {
        return ShippingUtils.getLastModified(request);
    }
}

